//////////////////////////////////////////////////////////////////////////////
//                      Copyright (c) 2005 Gerhard Schalk
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Projekt:       Texas Instrument I2C Digital Temperatur Sensor TMP100  
//
//   Filename:      TempSensor_TMP100.C
//////////////////////////////////////////////////////////////////////////////   

//-------------------------------------------------------------------------
//  Include Dateien
//-------------------------------------------------------------------------
#include <stdio.h>          // Standard I/O Funktionen
#include "REG517A.H"
#include "I2C.H"
#include "TempSensor_TMP100.H"



//--------------------------------------------------------------------------
// Funktion: char EnableTMP100_12BitMode(void)
//
// TMP100 in 12 Bit Modus setzen
//--------------------------------------------------------------------------
char EnableTMP100_12BitMode(void)
{
    unsigned char ackValueRD;

    // Config. TMP100 in 12-Bit Mode
    i2c_Start();                                    // I2C Start Condition
    ackValueRD = i2c_MasterWrite(TMP100_I2C_WRAdr); // WR: Slave WR Adress
    if(ackValueRD)          
    {   
       return ERROR; 
    }    
    ackValueRD = i2c_MasterWrite(0x01);   // Pointer Register = Config. Register
    ackValueRD = i2c_MasterWrite(0x60);   // Umschalten auf 12-Bit Auflsung 
    i2c_Stop();                           // I2C Stop Condition
    return OK;
}

//--------------------------------------------------------------------------
// Funktion: char ReadTempTMP100(float *TempValue)
//
// TMP100 Temperaturwert lesen
//--------------------------------------------------------------------------
char ReadTempTMP100(float *TempValue)
{
    unsigned char ackValueRD;
    int TempRegValue;

    // Lesen des  Temperaturwertes 
    i2c_Start();                                    // I2C Start Condition
    ackValueRD = i2c_MasterWrite(TMP100_I2C_WRAdr); // WR: Slave WR Adress
    if(ackValueRD)          
    {   
       return ERROR; 
    }    
    ackValueRD = i2c_MasterWrite(0x00);             // Pointer Register = Temp. Register
    i2c_Start();                                    // I2C Start Condition
    ackValueRD = i2c_MasterWrite(TMP100_I2C_RDAdr); // WR: Slave RD Temp. Register Byte 1
    TempRegValue = i2c_MasterRead (ACK);            // RD: Temp. High Byte
    TempRegValue <<=8;
    TempRegValue |= i2c_MasterRead (NACK);          // WR: Slave RD Temp. Register Byte 2
    i2c_Stop();                                     // I2C Stop Condition
    
    // Berechnen der Temperatur in Grad Celsius
    *TempValue = (float)((TempRegValue * 128.0) / 32768.0);
    return OK;
}



