//////////////////////////////////////////////////////////////////////////////
//                      Copyright (c) 2005 Gerhard Schalk
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Projekt:    Temperatur Sensoren (TMP100, DS1822, NTC) und RTC
//               
//               Funktionsauswahl via Tasten:
//               Taste 0 .. TMP100 
//               Taste 1 .. DS1822
//               Taste 3 .. NTC
//               Taste 4 .. Uhrzeit setzen
//
//               Funktionsauswahl via Drehimpulsgeber:
//               Drehrichtung des Drehimpulsgeber .. TMP100 / DS1822 / NTC       
//               Push Botton (Drehimpulsgeber) .. Uhrzeit setzen
//
//   Filename:   MAIN.C               
//////////////////////////////////////////////////////////////////////////////   

//-------------------------------------------------------------------------
//  Include Dateien
//-------------------------------------------------------------------------
#include "REG517A.H"            // 80C517 SFR-Register
#include <stdio.h>              // Standard I/O Funktionen
#include <80C517.H>             // Spezielle 80C517 Library Funktionen

#include "lcd.h"                // LCD Funktionen 
#include "TempSensor_TMP100.H"  // Temperatursensor TMP100 
#include "TempMeasure.H"        // Temperatursenosr Demos
#include "RTCWatch.H"           // RTC
#include "Drehgeber.H"          // Drehimpulsgegber

//-------------------------------------------------------------------------
//                                Defines
//-------------------------------------------------------------------------
#define   TMP100       0
#define   DS1822       1
#define   NTC          2
#define   SetRTCWatch  3


//-------------------------------------------------------------------------
//                              Hauptprogramm
//-------------------------------------------------------------------------
void main(void)
{   
    //  Lokale Variablen 
    unsigned char TempSensorSelect;
    unsigned char DrehimpulsWert;

    P6 = 0x40;    
    P4 = 0xFF;                  // 7-Segment aus

    P6 = 0x80;
    P4 = 0x00;                  // LED-Balken aus

    init_lcd();                 // LCD initialisieren
    blank_lcd();                // LCD Anzeige lschen

    init_Drehimpulsgeber();      // Drehimpulsgeber initialisieren

    EnableTMP100_12BitMode();   // TMP100 in 12 Bit Modus setzen

    TempSensorSelect = TMP100;

    SetWatch(1,13);             // Uhrzeit setzte   
 
    while(1)
    {
       DisplayWatch(1,13);      // Aktuelle Uhrzeit anzeigen

       // Funktionsauswahl via Tastatur 
       if(KeySignal==0)         // Taste gedrueckt? (Low aktive)
       {
          TempSensorSelect = KeyValue;
          while(KeySignal==0);  // Warten bis Taste wieder losgelassen wurde
       }

       // Funktionsauswahl via Drehimpulsgeber
       DrehimpulsWert = TempSensorSelect;
       DrehimpulsWert = Drehimpulsgeber(DrehimpulsWert);
       
       if((DrehimpulsWert >= 0) && (DrehimpulsWert <= 2))
           TempSensorSelect = DrehimpulsWert;

       //  Auswerten des Push Botton (Drehimpulsgeber)
       if(PushBotton)
            SetWatch(1,13);             // Uhrzeit setzte    

   
       switch(TempSensorSelect)
       {
                            // TMP100 Temp. Wert Anzeigen
            case TMP100:    MeasureTMP100();        
                            break;

                            // DS1822 Temp. Wert Anzeigen
            case DS1822:    MeasureDS1822();        
                            break;     

                            // NTC Temp. Wert Anzeigen
            case NTC:       MeasureNTC();          
                            break;
                            
                            // Uhrzeit setzten
            case SetRTCWatch: SetWatch(1,13);       
                              TempSensorSelect = TMP100; 
                              break;

                            // TMP100 Temp. Wert Anzeigen
            default:        TempSensorSelect = TMP100; 
                            break; 
        }
    }

}  // Ende des Hauptprogrammes	


