//////////////////////////////////////////////////////////////////////////////
//                      Copyright (c) 2005 Gerhard Schalk
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Header File:   Dallas DS1822 1-Wire Digital Thermometer
//////////////////////////////////////////////////////////////////////////////  
#ifndef __DS1822_H_INCLUDED
#define __DS1822_H_INCLUDED

    //--------------------------------------------------------------------------
    // DS1822 Hardware I/O Port Defines
    //--------------------------------------------------------------------------
    sbit        DQ = P5^0;
    #define     HIGH	1         	
    #define		LOW		0	    
    
    //--------------------------------------------------------------------------
    // DS1822 Command Definitionen
    //--------------------------------------------------------------------------
    #define     DS1822_Present     0
    #define     DS1822_NotPresent  1
    
    // DS1822 Funktions - Kommandosbytes
    #define     ReadROMCommando 0x33
    #define     SkipROM         0xCC
    #define     ConvertTemp     0x44
    #define     ReadScratchpad  0xBE
    #define     WriteScratchpad 0x4E
    #define     CopyScratchpad  0x48
    #define     RecallEE        0xB8
    #define     ReadPowerSupply 0xB4


    //--------------------------------------------------------------------------
    //  DS1822 User Funktionen
    //--------------------------------------------------------------------------
    #define OK      0
    #define ERROR  -1        

    char DS1822ReadROMCode(unsigned char *RomCodeData); // Lesen der ROM Daten
    char DS1822ReadTemp(float *Temp);   // Aktuelle Temperatur auslesen 

    //--------------------------------------------------------------------------
    //  DS1822 Debug Funktion(en)
    //--------------------------------------------------------------------------
    char DS1822ReadScratchpadRAM(void); // Auslesen aller RAM Register


    //--------------------------------------------------------------------------
    // Interne Dallas 1-Wire Signaling Funktionen
    //--------------------------------------------------------------------------
    void OneWireDelay(int time);    
    unsigned char OneWire_Reset(void);

    unsigned char OneWireReadBit(void);
    void OneWireWriteBit(char bitValue);

    unsigned char OneWireReadByte(void);
    void OneWireWriteByte(char byteVal);


    
    

#endif
