//////////////////////////////////////////////////////////////////////////////
//                      Copyright (c) 2005 Gerhard Schalk
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Projekt:       I2C Single Master Bitlevel-Schicht Funktionen 
//
//   Filename:      I2C.C
//////////////////////////////////////////////////////////////////////////////   

//-------------------------------------------------------------------------
//  Include Dateien
//-------------------------------------------------------------------------
#include <intrins.h>
#include "REG517A.H"
#include "I2C.h"


#define DelayTime     DelayNormalMode 

void I2CDelay(msec)
{
  data char i;
    for (msec; msec!=0; msec--) 
        for (i=0; i<83; i++)
            ;
}


//-------------------------------------------------------------------------
//  Funktion:   void i2c_Start (void)
//  Setzt den Bus zuerst in den Ruhezustand und 
//  erzeugt eine I2C Start Condition
//-------------------------------------------------------------------------
void i2c_Start (void)
{
    // SDA und SCL in den Ruhezustand setzen
	SDA = HIGH;							
	SCL = HIGH;	
    I2CDelay(DelayTime);

    // Start Condition
	SDA = LOW;
    I2CDelay(DelayTime);
    SCL = LOW;								
    I2CDelay(DelayTime);	
}

//-------------------------------------------------------------------------
//  Funktion:   void i2c_Stop (void)
//  Erzeugt eine I2C Stop Condition   
//-------------------------------------------------------------------------
void i2c_Stop (void)
{
	SCL = LOW;
	SDA = LOW;	

    // Stop Condition
    I2CDelay(DelayTime);							
	SCL = HIGH;	
    I2CDelay(DelayTime);													
	SDA = HIGH;
    I2CDelay(DelayTime);														
}

//-------------------------------------------------------------------------
//  Funktion: char i2c_MasterWrite (unsigned char output_data)
//  Input:    I2C Output Data
//  Return:   0..ACK / 1..NACK .. vom Slave
//  Schreibt Daten in einen I2C Slave
//-------------------------------------------------------------------------
unsigned char i2c_MasterWrite (unsigned char output_data)
{
	unsigned char i;
    unsigned char mask = 0x80;

    // 8 Data Bits via I2C senden (MSB first)
	for(i = 0; i < 8; i++) 
	{
		SDA = (output_data & mask) ? (bit)1 : (bit)0;  // Datenbyte ausgeben
      	mask  >>= 1;                           		   // Datenbyte schieben
	    I2CDelay(DelayTime);							
        SCL = HIGH;   		                  		   // I2C CLK ausgeben 
        I2CDelay(DelayTime);							
        SCL = LOW;	
	}
    
    // Acknowledge vom Slave lesen
	
    SDA =   HIGH;           // SDA in Input Mode 
    I2CDelay(DelayTime);
    SCL = HIGH;             // I2C CLK ausgeben
    i =   SDA;              // ACK/NACK vom Slave lesen 
    I2CDelay(DelayTime);							
    SCL = LOW;
    I2CDelay(DelayTime);
    return i;   // Return Acknowledge Status 0..ACK (OK), 1..NAK
}

//-------------------------------------------------------------------------
//  Funktion: unsigned char i2c_MasterRead (bit ACKValue)
//  Input:    Master ACK Value 
//  Return:   Input Datenbyte
//  Liest Daten von einen I2C Slave
//-------------------------------------------------------------------------
unsigned char i2c_MasterRead (bit ACKValue)
{
	unsigned char i;
    unsigned char input_data = 0;
    unsigned char mask = 0x80;
    
    
    // 8 Data Bits via I2C lesen (MSB first)
	for(i = 0; i < 8; i++)  	            
	{
		SCL = HIGH;           // I2C CLK HIGH ausgeben 
        if(SDA==HIGH)               // Datenbit lesen und speichern  
            input_data |= mask;
        mask >>= 1;        
    	SCL = LOW;			  // I2C CLK LOW ausgeben 
        I2CDelay(DelayTime);							 
	}

    // Acknowledge Slave
    SDA = ACKValue;          // ACK senden
	SCL = HIGH;              // I2C CLK HIGH ausgeben
	I2CDelay(DelayTime);							
    SCL = LOW;	             // I2C CLK LOW ausgeben
    SDA = HIGH;              // SDA in Input Mode 
    I2CDelay(DelayTime);
    return input_data;
}
