//////////////////////////////////////////////////////////////////////////////
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Projekt:  Verschiedene Routinen zur Ansteuerung eines Sharp LCD Displays
//             vom Typ LM20A21 (2 Zeilen, 20 Spalten) 
//   Filename: lcd.c              
////////////////////////////////////////////////////////////////////////////// 
#include "REG517A.H"
#include "lcd.h"  

#define SPALTEN 20
#define ZEILEN 2

// LCD-Anschluesse an Ports */

  sbit RS = P5^1;
  sbit RW = P5^2;
  sbit EN = P5^3;
  sbit D4 = P5^4;
  sbit D5 = P5^5;
  sbit D6 = P5^6;
  sbit D7 = P5^7;

// Wartet eine entsprechende Anzahl von Millisekunden 

void warte(unsigned int msec)

{ data char i;
  for (msec; msec!=0; msec--) for (i=0; i<83; i++);
}


// Wartet, bis LCD neue Daten aufnehmen kann 

void busy_lcd(void)
{   data bit busy;
    busy =1;
    while (busy == 1) 
    {
        D7=1; D6=1; D5=1; D4=1;      // Ports auf Eingabe
        RS=0; RW=1;
        EN=1; busy=D7; EN=0;         // lese 1.Halbbyte  
                                     // busy-flag in D7  
        EN=1; EN=0;                  // lese 2.Halbbyte  
    }
}

// Initialisiert den LCD-Controller auf 4 Bit Betrieb etc

void init_lcd(void)

{
  D7=1; D6=1; D5=1; D4=1;
  RS=0; RW=0; EN=0;
  warte (20);

  D7=0; D6=0; D5=1; D4=1; EN=1; EN=0;   // func set: interface is 8 bit long
  warte (5);

  D7=0; D6=0; D5=1; D4=1; EN=1; EN=0;   // func set: interface is 8 bit long
  warte (1);

  D7=0; D6=0; D5=1; D4=1; EN=1; EN=0;   // func set: interface is 8 bit long
  warte (1);

  D7=0; D6=0; D5=1; D4=0; EN=1; EN=0;   // func set: interface is 4 bit long
  warte (1);

  D7=0; D6=0; D5=1; D4=0; EN=1; EN=0;   // func set: interface is 4 bit long
  D7=1; D6=0; D5=0; D4=0; EN=1; EN=0;   // number of display lines is 2    

  busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;    // D6=1 display on;
  D7=1; D6=1; D5=0; D4=0; EN=1; EN=0;

  busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;    // clear display 
  D7=0; D6=0; D5=0; D4=1; EN=1; EN=0;

  busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;    // entry mode: decrement, no shift 
  D7=0; D6=1; D5=0; D4=0; EN=1; EN=0;
}


//------------------------------------------------------------------------
// Funktion: void CursorBlinkOnOff(char zeile, char spalte, bit Status)
//------------------------------------------------------------------------ 
void CursorBlinkOnOff(char zeile, char spalte, bit Status)
{
    int i, CursorPos;    
    
    if(Status)
    {   
        //  Write Commando: Set Cursor to home position
        busy_lcd ();
        RS=0; RW=0;               // RS = Commando / R/W = Write
        D7=0; D6=0; D5=0; D4=0;   // High Nibble des Commando Byte
        EN=1; EN=0;               // Enable - Datenuebernahme          
        D7=0; D6=0; D5=1; D4=0; 
        EN=1; EN=0;               // Low Nibble des Commando Byte

        //  Move cursor to position
        CursorPos = spalte-1 + (zeile-1)*40;      
        for(i = 0; i < CursorPos ; i++)
        {
            // Write Commando: Move Cursor
            busy_lcd ();
            RS=0; RW=0;
            D7=0; D6=0; D5=0; D4=1; 
            EN=1; EN=0;    
            D7=0; D6=1; D5=0; D4=0;   
            EN=1; EN=0;
        }   
        
        // Write Commando: Cursor Blink ON
        busy_lcd ();
        RS=0; RW=0;
        D7=0; D6=0; D5=0; D4=0; 
        EN=1; EN=0;   
        D7=1; D6=1; D5=0; D4=1;     // D4=1 blink of cursor ON 
        EN=1; EN=0;
  }
  else  
  { 
    // Write Commando: Cursor Blink OFF  
    busy_lcd ();
    RS=0; RW=0;
    D7=0; D6=0; D5=0; D4=0; 
    EN=1; EN=0;  
    D7=1; D6=1; D5=0; D4=0;     // D4=1 blink of cursor OFF 
    EN=1; EN=0;
  }

}

// Loescht die Anzeige und stellt den Cursor zurueck
void blank_lcd(void)          
{ busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;   // clear display 
  D7=0; D6=0; D5=0; D4=1; EN=1; EN=0;
  busy_lcd ();
  RS=0; RW=0;
  D7=0; D6=0; D5=0; D4=0; EN=1; EN=0;   // return home
  D7=0; D6=0; D5=1; D4=0; EN=1; EN=0;
}

void char_lcd(char zeile, char spalte, char ascii)

{ data char adresse;

  if (zeile>0 && zeile<(ZEILEN+1))
     if (spalte>0 && spalte<(SPALTEN+1))
               { adresse = spalte-1 + (zeile-1)*64;
                 busy_lcd(); RS=0; RW=0;
                 D7 = 1;
                 D6 = adresse & 64;
                 D5 = adresse & 32;
                 D4 = adresse & 16;
                 EN=1; EN=0;
                 D7 = adresse & 8;
                 D6 = adresse & 4;
                 D5 = adresse & 2;
                 D4 = adresse & 1;
                 EN=1; EN=0;
                 busy_lcd(); RS=1; RW=0;
                 D7 = ascii & 128;
                 D6 = ascii & 64;
                 D5 = ascii & 32;
                 D4 = ascii & 16;
                 EN=1; EN=0;
                 D7 = ascii & 8;
                 D6 = ascii & 4;
                 D5 = ascii & 2;
                 D4 = ascii & 1;
                 EN=1; EN=0;
                }
}

void print_lcd(char zeile, char spalte, char *str)

   {
      char i;

      for ( i = spalte; (i <= 20) &&  *str; i++)

         {
            warte(10);
            char_lcd(zeile,i,*str);
            str++;
         };
    }

#if 0
void graf_lcd (char adresse, char muster)
{
        if (adresse <= 64) {              // Gueltiger CG-Bereich ?  
                busy_lcd (); RS=0; RW=0;  // CG-RAM-Adresse 0..64     
                D7 = 0;                   // oberes Halbbyte adresse 
                D6 = 1;
                D5 = adresse & 32;
                D4 = adresse & 16;
                EN=1; EN=0;
                D7 = adresse & 8;         // unteres Halbbyte / Adresse   
                D6 = adresse & 4;
                D5 = adresse & 2;
                D4 = adresse & 1;
                EN=1; EN=0;
                busy_lcd (); RS=1; RW=0;  // muster an CG-RAM
                D7 = muster & 128;        // oberes Halbbyte / Muster 
                D6 = muster & 64;
                D5 = muster & 32;
                D4 = muster & 16;
                EN=1; EN=0;
                D7 = muster & 8;          // unteres Halbbyte / Muster  
                D6 = muster & 4;
                D5 = muster & 2;
                D4 = muster & 1;
                EN=1; EN=0;
        }
}
#endif

/************************************************
*  Define special grafic symbols                *
*  def_graf()                                   *
*************************************************/
#if 0
void spezial_graf(void)
{

 /*** Symbol Nr. 0hex ***/
  graf_lcd ( 0, 0x00);      /*   . . . . .   */
  graf_lcd ( 1, 0x00);      /*   . . . . .   */
  graf_lcd ( 2, 0x00);      /*   . . . . .   */
  graf_lcd ( 3, 0x00);      /*   . . . . .   */
  graf_lcd ( 4, 0x00);      /*   . . . . .   */
  graf_lcd ( 5, 0x00);      /*   . . . . .   */
  graf_lcd ( 6, 0x00);      /*   . . . . .   */
  graf_lcd ( 7, 0x00);      /*   . . . . .   */

 /*** Symbol Nr. 1hex ***/
  graf_lcd ( 8, 0x00);      /*   . . . . .   */
  graf_lcd ( 9, 0x00);      /*   . . . . .   */
  graf_lcd (10, 0x04);      /*   . . * . .   */
  graf_lcd (11, 0x1E);      /*   * * * * .   */
  graf_lcd (12, 0x1F);      /*   * * * * *   */
  graf_lcd (13, 0x1E);      /*   * * * * .   */
  graf_lcd (14, 0x04);      /*   . . * . .   */
  graf_lcd (15, 0x00);      /*   . . . . .   */

 /*** Symbol Nr. 2hex ***/
  graf_lcd (16, 0x00);      /*   . . . . .   */
  graf_lcd (17, 0x00);      /*   . . . . .   */
  graf_lcd (18, 0x04);      /*   . . * . .   */
  graf_lcd (19, 0x0F);      /*   . * * * *   */
  graf_lcd (20, 0x1F);      /*   * * * * *   */
  graf_lcd (21, 0x0F);      /*   . * * * *   */
  graf_lcd (22, 0x04);      /*   . . * . .   */
  graf_lcd (23, 0x00);      /*   . . . . .   */

 /*** Symbol Nr. 3hex ***/
  graf_lcd (24, 0x00);      /*   . . . . .   */
  graf_lcd (25, 0x04);      /*   . . * . .   */
  graf_lcd (26, 0x0E);      /*   . * * * .   */
  graf_lcd (27, 0x1F);      /*   * * * * *   */
  graf_lcd (28, 0x0E);      /*   . * * * .   */
  graf_lcd (29, 0x0E);      /*   . * * * .   */
  graf_lcd (30, 0x0E);      /*   . * * * .   */
  graf_lcd (31, 0x00);      /*   . . . . .   */

 /*** Symbol Nr. 4hex ***/
  graf_lcd (32, 0x00);      /*   . . . . .   */
  graf_lcd (33, 0x0E);      /*   . * * * .   */
  graf_lcd (34, 0x0E);      /*   . * * * .   */
  graf_lcd (35, 0x0E);      /*   . * * * .   */
  graf_lcd (36, 0x1F);      /*   * * * * *   */
  graf_lcd (37, 0x0E);      /*   . * * * .   */
  graf_lcd (38, 0x04);      /*   . . * . .   */
  graf_lcd (39, 0x00);      /*   . . . . .   */

 /*** Symbol Nr. 5hex  representiert ENTER Taste    ***/
  graf_lcd (40, 0x1F);      /*   * * * * *   */
  graf_lcd (41, 0x01);      /*   . . . . *   */
  graf_lcd (42, 0x01);      /*   . . . . *   */
  graf_lcd (43, 0x05);      /*   . . * . *   */
  graf_lcd (44, 0x0D);      /*   . * * . *   */
  graf_lcd (45, 0x1F);      /*   * * * * *   */
  graf_lcd (46, 0x0C);      /*   . * * . .   */
  graf_lcd (47, 0x04);      /*   . . * . .   */

 /*** Symbol Nr. 6hex ***/
  graf_lcd (48, 0x04);      /*   . . * . .   */
  graf_lcd (49, 0x0E);      /*   . * * * .   */
  graf_lcd (50, 0x1F);      /*   * * * * *   */
  graf_lcd (51, 0x0E);      /*   . * * * .   */
  graf_lcd (52, 0x0E);      /*   . * * * .   */
  graf_lcd (53, 0x0E);      /*   . * * * .   */
  graf_lcd (54, 0x0E);      /*   . * * * .   */
  graf_lcd (55, 0x0E);      /*   . * * * .   */


   /*** Symbol Nr. 7hex ***/
  graf_lcd (56, 0x08);      /*   . * . . .   */
  graf_lcd (57, 0x14);      /*   * . * . .   */
  graf_lcd (58, 0x08);      /*   . * . . .   */
  graf_lcd (59, 0x03);      /*   . . . * *   */
  graf_lcd (60, 0x04);      /*   . . * . .   */
  graf_lcd (61, 0x04);      /*   . . * . .   */
  graf_lcd (62, 0x03);      /*   . . . * *   */
  graf_lcd (63, 0x00);      /*   . . . . .   */
  busy_lcd();
}
#endif

#if 0
void def_balken(void)
{

 /*** Symbol Nr. 0hex ***/
  graf_lcd ( 0, 0x00);      /*   . . . . .   */
  graf_lcd ( 1, 0x00);      /*   . . . . .   */
  graf_lcd ( 2, 0x00);      /*   . . . . .   */
  graf_lcd ( 3, 0x00);      /*   . . . . .   */
  graf_lcd ( 4, 0x00);      /*   . . . . .   */
  graf_lcd ( 5, 0x00);      /*   . . . . .   */
  graf_lcd ( 6, 0x00);      /*   . . . . .   */
  graf_lcd ( 7, 0x00);      /*   . . . . .   */

 /*** Symbol Nr. 1hex ***/
  graf_lcd ( 8, 0x10);      /*   * . . . .   */
  graf_lcd ( 9, 0x10);      /*   * . . . .   */
  graf_lcd (10, 0x10);      /*   * . . . .   */
  graf_lcd (11, 0x10);      /*   * . . . .   */
  graf_lcd (12, 0x10);      /*   * . . . .   */
  graf_lcd (13, 0x10);      /*   * . . . .   */
  graf_lcd (14, 0x10);      /*   * . . . .   */
  graf_lcd (15, 0x10);      /*   *. . . . .  */

 /*** Symbol Nr. 2hex ***/
  graf_lcd (16, 0x18);      /*   * * . . .   */
  graf_lcd (17, 0x18);      /*   * * . . .   */
  graf_lcd (18, 0x18);      /*   * * . . .   */
  graf_lcd (19, 0x18);      /*   * * . . .   */
  graf_lcd (20, 0x18);      /*   * * . . .   */
  graf_lcd (21, 0x18);      /*   * * . . .   */
  graf_lcd (22, 0x18);      /*   * * . . .   */
  graf_lcd (23, 0x18);      /*   * * . . .   */


 /*** Symbol Nr. 3hex ***/
  graf_lcd (24, 0x1C);      /*   * * * . .   */
  graf_lcd (25, 0x1C);      /*   * * * . .   */
  graf_lcd (26, 0x1C);      /*   * * * . .   */
  graf_lcd (27, 0x1C);      /*   * * * . .   */
  graf_lcd (28, 0x1C);      /*   * * * . .   */
  graf_lcd (29, 0x1C);      /*   * * * . .   */
  graf_lcd (30, 0x1C);      /*   * * * . .   */
  graf_lcd (31, 0x1C);      /*   * * * . .   */


 /*** Symbol Nr. 4hex ***/
  graf_lcd (32, 0x1E);      /*   * * * * .   */
  graf_lcd (33, 0x1E);      /*   * * * * .   */
  graf_lcd (34, 0x1E);      /*   * * * * .   */
  graf_lcd (35, 0x1E);      /*   * * * * .   */
  graf_lcd (36, 0x1E);      /*   * * * * .   */
  graf_lcd (37, 0x1E);      /*   * * * * .   */
  graf_lcd (38, 0x1E);      /*   * * * * .   */
  graf_lcd (39, 0x1E);      /*   * * * * .   */


 /*** Symbol Nr. 5hex ***/
  graf_lcd (40, 0x1F);      /*   * * * * *   */
  graf_lcd (41, 0x1F);      /*   * * * * *   */
  graf_lcd (42, 0x1F);      /*   * * * * *   */
  graf_lcd (43, 0x1F);      /*   * * * * *   */
  graf_lcd (44, 0x1F);      /*   * * * * *   */
  graf_lcd (45, 0x1F);      /*   * * * * *   */
  graf_lcd (46, 0x1F);      /*   * * * * *   */
  graf_lcd (47, 0x1F);      /*   * * * * *   */

 /*** Symbol Nr. 6hex ***/
  graf_lcd (48, 0x04);      /*   . . . . .   */
  graf_lcd (49, 0x04);      /*   * . . . .   */
  graf_lcd (50, 0x04);      /*   * * . . .   */
  graf_lcd (51, 0x04);      /*   * * * . .   */
  graf_lcd (52, 0x1F);      /*   * * * * .   */
  graf_lcd (53, 0x0E);      /*   * * * . .   */
  graf_lcd (54, 0x04);      /*   * * . . .   */
  graf_lcd (55, 0x00);      /*   * . . . .   */
  busy_lcd();
}
#endif
