//////////////////////////////////////////////////////////////////////////////
//                      Copyright (c) 2005 Gerhard Schalk
//
//                      80C537A Mikrocontroller Training
//                              C Programmierung
//
//////////////////////////////////////////////////////////////////////////////
//   Projekt:       RTC / Philips PCF8563 Funktionen 
//
//   Filename:      RTC_PCF8563.C
//////////////////////////////////////////////////////////////////////////////   

//-------------------------------------------------------------------------
//  Include Dateien
//-------------------------------------------------------------------------
#include <stdio.h>          // Standard I/O Funktionen
#include "REG517A.H"
#include "I2C.H"
#include "RTC_PCF8563.H"



//--------------------------------------------------------------------------
// Funktion: char ReadRTCTime(struct time *Time)
//--------------------------------------------------------------------------
char ReadRTCTime(struct time *Time)
{
    unsigned char ackValueRD;

    i2c_Start();                                     // I2C Start Condition
    ackValueRD = i2c_MasterWrite(PCF8563_I2C_WRAdr); // WR: Slave WR Adress
    if(ackValueRD)          
    {   
        // Keine ACK vom PCF8563 => Fehler   
       Time->sec = 0;
       Time->min = 0;
       Time->hour = 0;
       return ERROR; 
    }    
    ackValueRD = i2c_MasterWrite(0x02);               // WR: Wordadress
    i2c_Start();                                      // I2C Start Condition
    ackValueRD = i2c_MasterWrite(PCF8563_I2C_RDAdr);  // WR: Slave RD Adress
    Time->sec = i2c_MasterRead (ACK);                 // RD: SEK Daten Byte
    Time->min = i2c_MasterRead (ACK);                 // RD: MIN Daten Byte
    Time->hour = i2c_MasterRead (NACK);               // RD: STD Daten Byte
    i2c_Stop();                                       // I2C Stop Condition
    
    //  Ausblenden von unbenutzten Bits zur Vermeidung von Anzeigefehler
    Time->sec &= 0x7F;
    Time->min &= 0x7F;
    Time->hour &= 0x3F;      

    return OK;
}


//--------------------------------------------------------------------------
// Funktion: char SetRTCTime(struct time *Time)
//--------------------------------------------------------------------------
char SetRTCTime(struct time *Time)
{
    unsigned char ackValueRD;

    i2c_Start();
    ackValueRD = i2c_MasterWrite(PCF8563_I2C_WRAdr);
    ackValueRD = i2c_MasterWrite(0x02);
    ackValueRD = i2c_MasterWrite(Time->sec);     // Sekunden
    ackValueRD = i2c_MasterWrite(Time->min);     // Minuten
    ackValueRD = i2c_MasterWrite(Time->hour);     // Stunden

    i2c_Stop();
    return OK;
}

#if 0
char TestReadAllRegister(void)
{
    unsigned int i;
    unsigned char ackValueRD;
    unsigned char RegValues[20];

    i2c_Start();                                         // I2C Start Condition
    ackValueRD = i2c_MasterWrite(PCF8563_I2C_WRAdr);     // WR:  Slave WR Adress
    if(ackValueRD)
    { 
        // Keine ACK vom PCF8563 => Fehler 
        printf("Error no ACK from PCF8563\n");           
        return ERROR;                                    
    }   
    ackValueRD = i2c_MasterWrite(0x00);                  // WR: Wordadress
    i2c_Start();                                         // I2C Start Condition
    ackValueRD = i2c_MasterWrite(PCF8563_I2C_RDAdr);     // WR:  Slave RD Adress

    for(i=0; i<14; i++)                                 
    {
        RegValues[i] = i2c_MasterRead(ACK);              // RD: Register 0x00..0x0E
        
    }
    RegValues[++i] = i2c_MasterRead(NACK);               // RD: Register 0x0F
    i2c_Stop();                                          // I2C Stop Condition
    
    // Registerwerte via RS232 ausgeben
    for(i=0; i<15; i++)
        printf("Register: %2d Value: %02X Hex\n",(int)i,(int)RegValues[i]);

    return OK;
}
#endif
